package com.sh.game.script.system.map;

import com.sh.common.config.ConfigDataManager;
import com.sh.game.config.model.CallBackConfig;
import com.sh.game.config.model.DeliverConfig;
import com.sh.game.config.model.InstanceQuickConfig;
import com.sh.game.config.model.ItemConfig;
import com.sh.game.config.model.MapConfig;
import com.sh.game.config.model.MoYuSundryConfig;
import com.sh.game.config.model.NPCConfig;
import com.sh.game.config.model.SundryConfig;
import com.sh.game.config.model.UnionBossConfig;
import com.sh.game.config.model.VipConfig;
import com.sh.game.entity.Role;
import com.sh.game.entity.Union;
import com.sh.game.entity.system.SabacShare2Data;
import com.sh.game.entity.system.SystemDataManager;
import com.sh.game.extra.parambuilder.BufferParamBuilder;
import com.sh.game.proto.BagProtos;
import com.sh.game.proto.MapProtos;
import com.sh.game.proto.Sabac2Protos;
import com.sh.game.proto.BagProtos.BagItemChangeList;
import com.sh.game.proto.MapProtos.BossInfo;
import com.sh.game.proto.MapProtos.GetMiniMapInfoResponse;
import com.sh.game.proto.MapProtos.MapEventStateChangeResponse;
import com.sh.game.proto.MapProtos.ObjectMoveResponse;
import com.sh.game.proto.MapProtos.PositionChangeReason;
import com.sh.game.proto.MapProtos.ReliveResponse;
import com.sh.game.proto.MapProtos.ReliveType;
import com.sh.game.proto.MapProtos.TransferByDeliverExpenseState;
import com.sh.game.proto.Sabac2Protos.TransferSabac2PalaceState;
import com.sh.game.proto.ShareServer2Protos.CommonInfo;
import com.sh.game.share2.message.ReqGameCommonMessage;
import com.sh.game.shareServer.ClientMapManger;
import com.sh.game.system.CallBackInfo;
import com.sh.game.system.bag.Bag;
import com.sh.game.system.bag.BagManager;
import com.sh.game.system.bag.VehicleManager;
import com.sh.game.system.bag.message.ResBagItemChangedMessage;
import com.sh.game.system.buffer.BufferManager;
import com.sh.game.system.buffer.ReliveDeathBuffer;
import com.sh.game.system.chat.translate.TranslateUtil;
import com.sh.game.system.home.HomeManager;
import com.sh.game.system.hsmonster.HSMonsterManager;
import com.sh.game.system.instance.ArenaInstance;
import com.sh.game.system.instance.ChickenDinnerInstance;
import com.sh.game.system.instance.CrossServer3v3Instance;
import com.sh.game.system.instance.CrossServer5v5Instance;
import com.sh.game.system.instance.FantasyBattleInstance;
import com.sh.game.system.instance.HomeInstance;
import com.sh.game.system.instance.InstanceManager;
import com.sh.game.system.instance.NewUnionBossInstance;
import com.sh.game.system.instance.ReliveAreaInstance;
import com.sh.game.system.instance.SLDGSecretInstance;
import com.sh.game.system.instance.SLGInstance_Flag;
import com.sh.game.system.instance.SLGInstance_Neutral;
import com.sh.game.system.instance.SabacInstance;
import com.sh.game.system.instance.SabacPalaceInstance;
import com.sh.game.system.instance.SabacShare2Instance;
import com.sh.game.system.instance.SafeReliveInstance;
import com.sh.game.system.instance.TowerInstance;
import com.sh.game.system.map.GameInstance;
import com.sh.game.system.map.GameMap;
import com.sh.game.system.map.Grid;
import com.sh.game.system.map.MapManager;
import com.sh.game.system.map.MapObjectFactory;
import com.sh.game.system.map.Point;
import com.sh.game.system.map.message.ResGetMiniMapInfoMessage;
import com.sh.game.system.map.message.ResMapEventChangeMessage;
import com.sh.game.system.map.message.ResObjectChangePositionMessage;
import com.sh.game.system.map.message.ResReliveMessage;
import com.sh.game.system.map.obj.Actor;
import com.sh.game.system.map.obj.GuardPet;
import com.sh.game.system.map.obj.IMapObject;
import com.sh.game.system.map.obj.Monster;
import com.sh.game.system.map.obj.Performer;
import com.sh.game.system.map.obj.Pet;
import com.sh.game.system.map.obj.Player;
import com.sh.game.system.map.obj.RandomBox;
import com.sh.game.system.map.script.IMapScript;
import com.sh.game.system.map.script.IPlayerEnterMapScript;
import com.sh.game.system.player.RoleManager;
import com.sh.game.system.player.message.ResFirstDieMessage;
import com.sh.game.system.sabac2.Sabac2Manager;
import com.sh.game.system.task.message.ResFlyToGoalMessage;
import com.sh.game.system.union.UnionManager;
import com.sh.game.util.MessageUtil;
import com.sh.game.util.TimeUtil;
import com.sh.game.util.Utils;
import com.sh.game.world.ApplicationContext;
import com.sh.script.ScriptEngine;
import com.sh.script.annotation.Script;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Script
public class MapScript implements IMapScript {
   private static final Logger log = LoggerFactory.getLogger(MapScript.class);
   private static final int RELIVE_ITEM_SUNDRY = 201;

   public void randomMoonIsland(Player player) {
      int targetMapId = 0;
      if (ApplicationContext.getInstance().isShareServerOr2()) {
         if (ApplicationContext.getInstance().isShareServer()) {
            targetMapId = 1004;
         } else if (ApplicationContext.getInstance().isShareServer2()) {
            if (!RoleManager.getInstance().checkCSHJArmyInstanceNPC(player, 1)) {
               MessageUtil.sendTipMessage(858, player.getId());
               return;
            }

            targetMapId = 10040;
            if (player.getMapId() != targetMapId) {
               return;
            }
         }

         if (player.isDead()) {
            MessageUtil.sendTipMessage(859, player.getId());
         } else {
            GameMap playerMap = MapManager.getInstance().getMap(player);
            if (playerMap != null) {
               if (ApplicationContext.getInstance().isShareServer() && playerMap.getId() != 1003) {
                  MessageUtil.sendTipMessage(860, player.getId());
               } else {
                  GameMap map = MapManager.getInstance().getMap(targetMapId, 1);
                  if (map == null) {
                     log.info("地图不存在");
                  } else {
                     SundryConfig config = (SundryConfig)ConfigDataManager.getInstance().getById(SundryConfig.class, 844);
                     if (config == null) {
                        log.info("土著未配置");
                     } else {
                        int[] itemList = config.getEffect();
                        if (itemList.length < 2) {
                           log.info("土著配置错误");
                        } else {
                           Bag bag = player.getRole().getBag();
                           if (BagManager.getInstance().removeItem(1123, 0, bag, itemList[0], itemList[1])) {
                              if (targetMapId == 1004) {
                                 MapManager.getInstance().randomTransferMap(player, targetMapId, 1);
                              } else if (targetMapId == 10040) {
                                 int[] points = RoleManager.getSLDGSundryConfigEffects(162, new int[]{67, 78, 99, 59, 135, 83, 44, 116, 163, 118, 70, 158, 105, 186, 141, 160});
                                 if (points == null || points.length == 0 || points.length % 2 != 0) {
                                    points = new int[]{67, 78, 99, 59, 135, 83, 44, 116, 163, 118, 70, 158, 105, 186, 141, 160};
                                 }

                                 int length = points.length;
                                 int random = Utils.random(0, length / 2 - 1);
                                 int index = random * 2;
                                 MapManager.getInstance().changePos(player, points[index], points[index + 1], PositionChangeReason.Transfer, false);
                              }

                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public boolean canTransferByCallBack(Player player, boolean union, int mapId, int line, boolean isKuaFu) {
      return this.canEnterMap(player, mapId, line, union, false, isKuaFu, false);
   }

   public boolean transferByCallBack(Player player, CallBackInfo callBackInfo, boolean union) {
      CallBackConfig callBackConfig = (CallBackConfig)ConfigDataManager.getInstance().getById(CallBackConfig.class, callBackInfo.getMapId());
      if (!this.canEnterMap(player, callBackInfo.getMapId(), callBackInfo.getLineId(), union, true, false, true)) {
         return false;
      } else {
         boolean isDemonFiled = callBackConfig.getItemLevelId() != 0;
         boolean bNeedCost = true;
         if (callBackInfo.getMapId() == 100702) {
            if (player.getRole().getVipLevel() < 1) {
               MessageUtil.sendTipMessage(861, player.getId());
               return false;
            }

            bNeedCost = player.getRole().getVipLevel() == 1;
         } else if (callBackInfo.getMapId() == 100703) {
            if (player.getRole().getVipLevel() < 1) {
               MessageUtil.sendTipMessage(2238, player.getId());
               return false;
            }

            bNeedCost = player.getRole().getVipLevel() == 2;
         }

         GameMap map = MapManager.getInstance().getMap(player);
         if (map != null) {
            CallBackConfig config = (CallBackConfig)ConfigDataManager.getInstance().getById(CallBackConfig.class, map.getId());
            if (config != null) {
               if (config.getCallType() != 0 && config.getCallType() == callBackConfig.getCallType()) {
                  bNeedCost = false;
               }

               if (config.getMapId() == callBackConfig.getMapId() && callBackInfo.getLineId() == map.getLine()) {
                  bNeedCost = false;
               }
            }
         }

         if (bNeedCost) {
            List cost;
            if (!isDemonFiled) {
               cost = this.costDeliverItem(player, callBackConfig.getRequireItems(), callBackInfo.getMapId(), 1111);
            } else {
               cost = MapManager.getInstance().costDeliverItem(player, callBackConfig.getItemLevelId(), callBackConfig.getMapId(), (Map)null, 1111);
            }

            if (cost == null) {
               if (isDemonFiled) {
                  MessageUtil.sendTipMessage(862, player.getId());
                  return false;
               }

               if (callBackConfig.getRequireItems().size() == 1) {
                  Map.Entry<Integer, Integer> entry = (Map.Entry)callBackConfig.getRequireItems().entrySet().iterator().next();
                  ItemConfig itemConfig = (ItemConfig)ConfigDataManager.getInstance().getById(ItemConfig.class, entry.getKey());
                  MessageUtil.sendTipMessage(2659, player.getId(), new Object[]{itemConfig.getName(), entry.getValue()});
                  return false;
               }

               if (callBackConfig.getRequireItems().size() > 1) {
                  String msg = TranslateUtil.getTipInfo(2660, player.getId());

                  String str;
                  for(Iterator var16 = callBackConfig.getRequireItems().entrySet().iterator(); var16.hasNext(); msg = msg.concat(str)) {
                     Map.Entry<Integer, Integer> entry = (Map.Entry)var16.next();
                     ItemConfig itemConfig = (ItemConfig)ConfigDataManager.getInstance().getById(ItemConfig.class, entry.getKey());
                     str = String.format("%s*%d ", itemConfig.getName(), entry.getValue());
                  }

                  msg = msg.concat("[-]");
                  MessageUtil.sendTipMessage(msg, player.getId());
                  return false;
               }
            }

            if (cost != null && !cost.isEmpty()) {
               ResBagItemChangedMessage res = new ResBagItemChangedMessage();
               res.setBagItemChangeList(BagItemChangeList.newBuilder().addAllChangeList(cost).setLogType(1111).build());
               MessageUtil.sendMsg(res, player.getId());
            }
         }

         int curMapId = player.getMapId();
         if (curMapId == callBackInfo.getMapId()) {
            MapManager.getInstance().changePos(player, callBackInfo.getPoint().getX(), callBackInfo.getPoint().getY(), PositionChangeReason.Transfer, false);
         } else {
            HomeManager.getInstance().shortBagToHomeStoreHouse(player);
            MapManager.getInstance().changeMap(player, callBackInfo.getMapId(), callBackInfo.getLineId(), callBackInfo.getPoint().getX(), callBackInfo.getPoint().getY(), PositionChangeReason.Transfer, false, false, 0);
            VehicleManager.getInstance().changeMaps(player, 0, 0, player.getMapId() == callBackInfo.getMapId(), 4, union, 0, 0, (MapProtos.PositionChangeReason)null, callBackInfo);
         }

         return true;
      }
   }

   public boolean canEnterMap(Player player, int mapId, int line, boolean union, boolean sendTip, boolean isKuaFu, boolean doCallBackCondition) {
      if (!isKuaFu) {
         GameMap gameMap = MapManager.getInstance().getMap(mapId, line);
         if (gameMap == null) {
            return false;
         }
      }

      CallBackConfig callBackConfig = (CallBackConfig)ConfigDataManager.getInstance().getById(CallBackConfig.class, mapId);
      if (callBackConfig == null) {
         if (sendTip) {
            MessageUtil.sendTipMessage(863, player.getId());
         }

         return false;
      } else {
         if (callBackConfig.getType() != 0) {
            if (union && callBackConfig.getType() != 1 || !union && callBackConfig.getType() != 2) {
               if (sendTip) {
                  MessageUtil.sendTipMessage(863, player.getId());
               }

               return false;
            }

            GameMap map = MapManager.getInstance().getMap(player);
            if (map instanceof TowerInstance) {
               return false;
            }
         }

         int level = 10;
         MoYuSundryConfig levelConfig = (MoYuSundryConfig)ConfigDataManager.getInstance().getById(MoYuSundryConfig.class, 678);
         if (levelConfig != null) {
            level = levelConfig.getEffect()[0];
         }

         if (player.getLevel() < level) {
            if (sendTip) {
               MessageUtil.sendTipMessage(865, player.getId());
            }

            return false;
         } else {
            int openServerDay = TimeUtil.getOpenServerDay(player.getServerId());
            int minLv = 0;
            int maxLv = 0;
            int minRein = 0;
            int maxRein = 0;
            boolean configIncreaseRein = false;
            int[] increaseRein = callBackConfig.getIncreaseRein();
            if (increaseRein != null && increaseRein.length > 0) {
               for(int i = 0; i < increaseRein.length && openServerDay >= increaseRein[i]; i += 5) {
                  minRein = increaseRein[i + 1];
                  maxRein = increaseRein[i + 2];
                  minLv = increaseRein[i + 3];
                  maxLv = increaseRein[i + 4];
                  configIncreaseRein = true;
               }
            }

            MoYuSundryConfig config = (MoYuSundryConfig)ConfigDataManager.getInstance().getById(MoYuSundryConfig.class, 769);
            if (!doCallBackCondition && config != null && config.getEffect().length > 0) {
               if (configIncreaseRein) {
                  if (player.getRole().getReincarnation() + config.getEffect()[0] >= minRein && (maxRein <= 0 || maxRein > 0 && player.getRole().getReincarnation() <= maxRein)) {
                     return true;
                  }
               } else if (callBackConfig.getReincarnation().length > 0 && player.getRole().getReincarnation() + config.getEffect()[0] >= callBackConfig.getReincarnation()[0] && (callBackConfig.getReincarnation().length == 1 || callBackConfig.getReincarnation().length > 1 && player.getRole().getReincarnation() <= callBackConfig.getReincarnation()[1])) {
                  return true;
               }
            }

            if (configIncreaseRein) {
               Role role = player.getRole();
               if (player.getLevel() > maxLv) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(865, player.getId());
                  }

                  return false;
               }

               if (maxRein > 0 && role.getReincarnation() > maxRein) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(866, player.getId());
                  }

                  return false;
               }

               if (player.getLevel() < minLv) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(865, player.getId());
                  }

                  return false;
               }

               if (minRein > 0 && role.getReincarnation() < minRein) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(866, player.getId());
                  }

                  return false;
               }
            } else {
               String msg;
               if (player.getLevel() < callBackConfig.getLevel()) {
                  msg = String.format("当前等级不足%d级", callBackConfig.getLevel());
                  if (sendTip) {
                     MessageUtil.sendTopTipMessage(msg, 382, player.getId());
                  }

                  return false;
               }

               if (callBackConfig.getReincarnation().length == 1) {
                  if (player.getRole().getReincarnation() < callBackConfig.getReincarnation()[0]) {
                     msg = String.format("飞升等级不足%d", callBackConfig.getReincarnation()[0]);
                     if (sendTip) {
                        MessageUtil.sendTopTipMessage(msg, 382, player.getId());
                     }

                     return false;
                  }
               } else if (callBackConfig.getReincarnation().length > 1) {
                  msg = "[ff0000]需要飞升等级为";
                  boolean add = false;

                  for(int i = 0; i < callBackConfig.getReincarnation().length && (i <= 0 || callBackConfig.getReincarnation()[i] != callBackConfig.getReincarnation()[i - 1]); ++i) {
                     if (!add) {
                        add = true;
                     } else {
                        msg = msg.concat(" ～ ");
                     }

                     msg = msg.concat(String.format("%d", callBackConfig.getReincarnation()[i]));
                  }

                  msg = msg.concat("[-]");
                  if (!ArrayUtils.contains(callBackConfig.getReincarnation(), player.getRole().getReincarnation())) {
                     if (sendTip) {
                        MessageUtil.sendTipMessage(msg, player.getId());
                     }

                     return false;
                  }
               }
            }

            if (mapId == 100701) {
               if (player.getRole().getVipLevel() < 1) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(861, player.getId());
                  }

                  return false;
               }
            } else if (mapId == 100702) {
               if (player.getRole().getVipLevel() < 1) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(861, player.getId());
                  }

                  return false;
               }
            } else if (mapId == 100703) {
               if (player.getRole().getVipLevel() < 1) {
                  if (sendTip) {
                     MessageUtil.sendTipMessage(2238, player.getId());
                  }

                  return false;
               }
            } else if (player.getRole().getVipLevel() < callBackConfig.getVip()) {
               if (sendTip) {
                  MessageUtil.sendTipMessage(2661, player.getId(), new Object[]{callBackConfig.getVip()});
               }

               return false;
            }

            return true;
         }
      }
   }

   public void transferByDeliverConfig(Player player, int id, boolean useStone, int miniMapTeleportID, boolean follow) {
      if (player != null) {
         ResFlyToGoalMessage resFly = new ResFlyToGoalMessage();
         MessageUtil.sendMsg(resFly, player.getId());
         if (!player.isDead()) {
            if (VehicleManager.getInstance().isVehiclePassenger(player) && !follow) {
               MessageUtil.sendTipMessage("乘坐他人坐骑，无法主动传送", player.getId());
            } else if (RoleManager.getInstance().isGoldBodyState(player)) {
               MessageUtil.sendTipMessage("[ff0000]神佑状态下不可进行此操作[-]", player.getId());
            } else if (player.getRole().isInYaBiao()) {
               MessageUtil.sendTipMessage(867, player.getId());
            } else if (!BufferManager.getInstance().canDoing(player)) {
               MessageUtil.sendTipMessage(642, player.getId());
            } else {
               DeliverConfig deliverConfig = (DeliverConfig)ConfigDataManager.getInstance().getById(DeliverConfig.class, id);
               if (deliverConfig == null) {
                  log.error("传送配置错误{}", id);
               } else {
                  MapConfig config = (MapConfig)ConfigDataManager.getInstance().getById(MapConfig.class, deliverConfig.getMapId());
                  if (!VehicleManager.getInstance().checkEnterMap(config, player)) {
                     MessageUtil.sendTipMessage("[ff0000]战车不能进入副本地图[-]", player.getId());
                  } else if (deliverConfig.getServerType() != 0 && deliverConfig.getServerType() != ApplicationContext.getInstance().getServerType()) {
                     MessageUtil.sendTipMessage(2241, player.getId());
                     log.error("transferByDeliverConfig() role={}, id={}", player.getName(), id);
                  } else {
                     int curMapId = player.getMapId();
                     if (deliverConfig.getPreMapId() > 0 && curMapId != deliverConfig.getPreMapId()) {
                        MessageUtil.sendTipMessage(868, player.getId());
                     } else {
                        boolean isDemonFiled = deliverConfig.getItemLevel() != 0;
                        int mapId = deliverConfig.getMapId();
                        GameMap curMap = MapManager.getInstance().getMap(player);
                        if (curMap instanceof GameInstance) {
                           SundryConfig sundryConfig = (SundryConfig)ConfigDataManager.getInstance().getById(SundryConfig.class, 235);
                           if (sundryConfig == null) {
                              log.error("杂项配置缺少235!");
                           } else {
                              GameInstance instance = (GameInstance)curMap;
                              if (ArrayUtils.contains(sundryConfig.getEffect(), instance.getInstanceType())) {
                                 if (!(curMap instanceof ReliveAreaInstance)) {
                                    MessageUtil.sendTipMessage(776, player.getId());
                                    return;
                                 }

                                 if (mapId != curMapId) {
                                    MessageUtil.sendTipMessage(776, player.getId());
                                    return;
                                 }
                              }
                           }
                        }

                        GameMap map = MapManager.getInstance().getMap(mapId, player.getLine());
                        Union union = UnionManager.getInstance().getUnionById(player.getRole().getUnionId());
                        if (mapId == 71013) {
                           if (union == null) {
                              MessageUtil.sendTipMessage(75, player.getId());
                              return;
                           }

                           map = MapManager.getInstance().getMap(mapId, union.getNewUnionBossLine());
                        }

                        if (map == null) {
                           map = MapManager.getInstance().getMap(mapId, 1);
                        }

                        if (map == null) {
                           log.error("试图传送到不存在的地图:{}", mapId);
                        } else {
                           if (map instanceof GameInstance) {
                              long ownerId = 0L;
                              boolean isSameMap = false;
                              if (map instanceof HomeInstance) {
                                 HomeInstance homeInstance = (HomeInstance)map;
                                 if (homeInstance != null) {
                                    ownerId = homeInstance.getOwner();
                                    if (ownerId == player.getId() && mapId == curMapId && homeInstance.getLine() == curMap.getLine()) {
                                       isSameMap = true;
                                    }
                                 }
                              }

                              boolean b = InstanceManager.getInstance().checkCanEnter(player, ((GameInstance)map).getMapId(), true, (InstanceQuickConfig)null, false, ownerId, isSameMap);
                              if (!b) {
                                 return;
                              }
                           }

                           int x;
                           if (ApplicationContext.getInstance().isShareServer2()) {
                              int[] effects = RoleManager.getSLDGSundryConfigEffects(0, new int[]{10041, 3, 10042, 4, 10043, 2, 10044, 1});
                              if (effects != null && effects.length != 0 && effects.length % 2 == 0) {
                                 Map<Integer, Integer> mapIdAndShare2UnionType = Utils.parseArrayToMap(effects);
                                 if (mapIdAndShare2UnionType.containsKey(mapId)) {
                                    Integer mapUnionType = (Integer)mapIdAndShare2UnionType.get(mapId);
                                    int unionTypeOfShare2 = player.getRole().getUnionTypeOfShare2();
                                    x = 0;
                                    SabacShare2Data sabacShare2Data = (SabacShare2Data)SystemDataManager.getInstance().getSystemData(SabacShare2Data.class);
                                    if (sabacShare2Data != null) {
                                       x = sabacShare2Data.getLeaderUnionType();
                                    }

                                    boolean isCost = true;
                                    if (x != 0 && unionTypeOfShare2 == x) {
                                       isCost = false;
                                    } else if (unionTypeOfShare2 == mapUnionType) {
                                       isCost = false;
                                    }

                                    if (isCost) {
                                       int costNumber = RoleManager.getSLDGSundryConfigEffect(157, 0, 100);
                                       if (costNumber <= 0) {
                                          costNumber = 100;
                                       }

                                       if (!player.getRole().getBag().checkItemCount(4, costNumber, false)) {
                                          MessageUtil.sendTopTipMessage("绑魔不足无法进入秘境", 495, player.getId());
                                          return;
                                       }

                                       if (!BagManager.getInstance().removeItem(1050, 0, player.getRole().getBag(), 4, costNumber)) {
                                          return;
                                       }
                                    }
                                 }
                              }
                           }

                           if (useStone) {
                              if (player.getRole().getBag().countItem(52000002) < 1L) {
                                 MessageUtil.sendTipMessage(857, player.getId());
                                 return;
                              }
                           } else if (deliverConfig.getNpcId() > 0 && !MapManager.getInstance().checkTaskNPCPosition(player, deliverConfig.getNpcId())) {
                              MessageUtil.sendTipMessage(870, player.getId());
                              return;
                           }

                           int checkDay = TimeUtil.getOpenServerDay(player.getServerId()) - deliverConfig.getTimeOpen();
                           if (checkDay < 0) {
                              MessageUtil.sendTipMessage(871, player.getId(), new Object[]{String.valueOf(-checkDay)});
                           } else {
                              int index = this.checkDeliverRequire(player, deliverConfig.getLevel(), deliverConfig.getItem(), deliverConfig.getVip(), 1, false);
                              if (index == 0) {
                                 index = this.checkDeliverRequire(player, deliverConfig.getLevel2(), deliverConfig.getItem2(), deliverConfig.getVip2(), 2, false);
                              }

                              if (index == 0) {
                                 this.checkDeliverRequire(player, deliverConfig.getLevel(), deliverConfig.getItem(), deliverConfig.getVip(), 1, true);
                              } else {
                                 List<BagProtos.BagItemInfo> cost = null;
                                 MapProtos.TransferByDeliverExpenseState state = this.checkNeedItem(player, mapId, id);
                                 if (state == TransferByDeliverExpenseState.NeedItem) {
                                    Map<Integer, Integer> costItem = new HashMap();
                                    if (index == 1) {
                                       if (!isDemonFiled) {
                                          cost = this.costDeliverItem(player, deliverConfig.getItem(), mapId, 1050);
                                          costItem = deliverConfig.getItem();
                                       } else {
                                          cost = MapManager.getInstance().costDeliverItem(player, deliverConfig.getItemLevel(), mapId, (Map)costItem, 1050);
                                       }
                                    } else if (!isDemonFiled) {
                                       cost = this.costDeliverItem(player, deliverConfig.getItem2(), mapId, 1050);
                                       costItem = deliverConfig.getItem2();
                                    } else {
                                       cost = MapManager.getInstance().costDeliverItem(player, deliverConfig.getItemLevel(), mapId, (Map)costItem, 1050);
                                    }

                                    if (cost != null) {
                                       Sabac2Manager.getInstance().updateSacbacReward((Map)costItem, mapId, id);
                                    }
                                 } else if (state == TransferByDeliverExpenseState.NoItem) {
                                    cost = new ArrayList();
                                 } else if (state != TransferByDeliverExpenseState.DataError && state == TransferByDeliverExpenseState.NoInto) {
                                    MessageUtil.sendTipMessage(872, player.getId());
                                    return;
                                 }

                                 if (cost == null) {
                                    MessageUtil.sendTipMessage(862, player.getId());
                                 } else {
                                    if (useStone) {
                                       List<BagProtos.BagItemInfo> list = player.getRole().getBag().removeItem(1050, 0, 52000002, 1);
                                       if (list == null) {
                                          log.error("消耗传送石错误");
                                       } else {
                                          ((List)cost).addAll(list);
                                       }
                                    }

                                    if (!((List)cost).isEmpty()) {
                                       ResBagItemChangedMessage res = new ResBagItemChangedMessage();
                                       res.setBagItemChangeList(BagItemChangeList.newBuilder().addAllChangeList((Iterable)cost).setLogType(1009).build());
                                       MessageUtil.sendMsg(res, player.getId());
                                    }

                                    x = deliverConfig.getX();
                                    int y = deliverConfig.getY();
                                    int toNpc = deliverConfig.getToNpc();
                                    if (x == 0 && y == 0 && toNpc > 0) {
                                       NPCConfig npcConfig = (NPCConfig)ConfigDataManager.getInstance().getById(NPCConfig.class, toNpc);
                                       if (npcConfig == null) {
                                          log.error("NPC配置错误:{}", toNpc);
                                          return;
                                       }

                                       x = npcConfig.getBornX();
                                       y = npcConfig.getBornY();
                                    }

                                    Grid grid;
                                    if (x == 0 && y == 0 && toNpc == 0) {
                                       grid = (Grid)Utils.randomChoose(map.getMapInfo().getBornPointList());
                                       if (grid == null) {
                                          log.error("transferByDeliverConfig() grid is null! id={}", id);
                                          return;
                                       }

                                       x = grid.getX();
                                       y = grid.getY();
                                    }

                                    if (deliverConfig.getRandom() == 1) {
                                       grid = MapManager.getInstance().randomAvailablePoint(map, x, y, 100, false, true);
                                       if (grid == null) {
                                          log.error("随机传送没有可用的点");
                                          return;
                                       }

                                       x = grid.getX();
                                       y = grid.getY();
                                    }

                                    if (curMapId == mapId) {
                                       MapManager.getInstance().changePos(player, x, y, PositionChangeReason.Transfer, deliverConfig.getSafe() == 1);
                                    } else {
                                       if (deliverConfig.getId() == 1003 || deliverConfig.getId() == 1004) {
                                          HomeManager.getInstance().shortBagToHomeStoreHouse(player);
                                       }

                                       MapManager.getInstance().changeMap(player, mapId, map.getLine(), x, y, PositionChangeReason.Transfer, true, deliverConfig.getSafe() == 1, miniMapTeleportID);
                                       VehicleManager.getInstance().changeMaps(player, id, 0, player.getMapId() == mapId, 1, useStone, 0, 0, (MapProtos.PositionChangeReason)null, (CallBackInfo)null);
                                    }

                                 }
                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public void relive(Player player, MapProtos.ReliveType type, boolean useYuanbao, boolean chagePos) {
      log.info("relive() player={}, id={}, type={}, userYuanBao={}", new Object[]{player.getName(), player.getId(), type, useYuanbao});
      if (!player.isDead()) {
         log.info("player={}, id={}, not dead()!", player.getName(), player.getId());
      } else {
         boolean isReliveBufferType = type == ReliveType.ReliveBuffer;
         if (!isReliveBufferType) {
            BufferManager.getInstance().removeBufferByClass(player, new Class[]{ReliveDeathBuffer.class});
         }

         if (player.getRole().getDieTimes() == 1 && player.getRole().getRechargeTimes() == 0) {
            MessageUtil.sendMsg(new ResFirstDieMessage(), player.getId());
         }

         GameMap map = MapManager.getInstance().getMap(player);
         if (map == null) {
            log.info("player={}, id={}, map is null!", player.getName(), player.getId());
         } else {
            if (!isReliveBufferType && (map instanceof SabacInstance || map instanceof SabacPalaceInstance || map instanceof CrossServer5v5Instance || map instanceof CrossServer3v3Instance || map instanceof SabacShare2Instance || map instanceof ChickenDinnerInstance || map instanceof ArenaInstance || map instanceof SafeReliveInstance || map instanceof ReliveAreaInstance || map instanceof SLGInstance_Neutral || map instanceof SLGInstance_Flag || map instanceof FantasyBattleInstance)) {
               GameInstance instance = (GameInstance)map;
               if (instance.relive(player, type) < 0) {
                  log.info("player={}, id={}, sabac!", player.getName(), player.getId());
                  return;
               }
            }

            if (map instanceof SLDGSecretInstance) {
               SLDGSecretInstance instance = (SLDGSecretInstance)map;
               instance.afterRelive(player);
            }

            MapConfig mapConfig = (MapConfig)ConfigDataManager.getInstance().getById(MapConfig.class, map.getId());
            int[] reliveCoord = mapConfig.getReliveCoord();
            if (type == ReliveType.InPlace || type == ReliveType.ReliveBuffer || !player.bePrisoner(false)) {
               if (type == ReliveType.BornPoint) {
                  player.setHp(player.getFinalAttribute().getMaxHp(), 8);
                  player.setInnerPower(player.getFinalAttribute().getMaxInnerPower());
                  player.setMp(player.getFinalAttribute().getMaxMp());
                  if (reliveCoord[1] == 0 && reliveCoord[2] == 0) {
                     map.reliveRandomPoint(player);
                  } else {
                     map.reliveBornPoint(player);
                  }
               } else if (type == ReliveType.InPlace) {
                  Role role = player.getRole();
                  Bag bag = role.getBag();
                  role.checkDayPassed();
                  SundryConfig sundryConfig = (SundryConfig)ConfigDataManager.getInstance().getById(SundryConfig.class, 201);
                  if (sundryConfig == null) {
                     log.error("杂项表{}复活道具未配置", 201);
                     return;
                  }

                  int[] itemId = sundryConfig.getEffect();
                  int itemNum = (int)bag.countItem(itemId[0]);
                  int usedNum = role.getUsedReliveNumber();
                  VipConfig vipConfig = (VipConfig)ConfigDataManager.getInstance().getById(VipConfig.class, role.getVipLevel());
                  if (vipConfig == null) {
                     log.error("vip{}未配置", role.getVipLevel());
                     return;
                  }

                  role.updateReliveNumber(vipConfig.getReborn(), usedNum, itemNum);
                  int reliveNumber = role.getReliveNumber();
                  if (reliveNumber <= 0 && !useYuanbao) {
                     MessageUtil.sendTipMessage(874, role.getId());
                  } else {
                     if (usedNum >= vipConfig.getReborn() && !useYuanbao) {
                        if (!BagManager.getInstance().removeItem(1044, 0, bag, itemId[0], 1)) {
                           MessageUtil.sendTipMessage(873, player.getId());
                           return;
                        }

                        role.setReliveNumber(reliveNumber - 1);
                        if (vipConfig.getReborn() > usedNum) {
                           role.setUsedReliveNumber(usedNum + 1);
                        }
                     } else if (useYuanbao) {
                        if (!role.getBag().reduceYuanbao(50000, 1044, 0)) {
                           return;
                        }
                     } else {
                        role.setReliveNumber(reliveNumber - 1);
                        if (vipConfig.getReborn() > usedNum) {
                           role.setUsedReliveNumber(usedNum + 1);
                        }
                     }

                     player.setHp(player.getFinalAttribute().getMaxHp(), 8);
                     player.setInnerPower(player.getFinalAttribute().getMaxInnerPower());
                     player.setMp(player.getFinalAttribute().getMaxMp());
                     MapManager.getInstance().afterRelive(map, player, type);
                     player.bePrisoner(true);
                  }
               } else if (type == ReliveType.LastMap) {
                  player.setHp(player.getFinalAttribute().getMaxHp(), 8);
                  player.setInnerPower(player.getFinalAttribute().getMaxInnerPower());
                  player.setMp(player.getFinalAttribute().getMaxMp());
                  MapManager.getInstance().afterRelive(map, player, type);
                  int mapId = reliveCoord[0];
                  int toX = reliveCoord[1];
                  int toY = reliveCoord[2];
                  if (reliveCoord.length > 3) {
                     for(int i = 0; i + 3 < reliveCoord.length; i += 4) {
                        if (player.getRole().getUnionTypeOfShare2() == reliveCoord[i]) {
                           mapId = reliveCoord[i + 1];
                           toX = reliveCoord[i + 2];
                           toY = reliveCoord[i + 3];
                        }
                     }
                  }

                  if (chagePos) {
                     if (map.getId() == mapId) {
                        if (toX == 0 && toY == 0) {
                           map.reliveRandomPoint(player);
                           return;
                        }

                        MapManager.getInstance().changePos(player, toX, toY, PositionChangeReason.Relive, false);
                     } else {
                        MapManager.getInstance().changeMap(player, mapId, 1, toX, toY, PositionChangeReason.Relive, true, false, 0);
                     }
                  }
               } else if (type == ReliveType.ReliveBuffer) {
                  ReliveDeathBuffer buffer = (ReliveDeathBuffer)BufferManager.getInstance().getBufferOfType(player, ReliveDeathBuffer.class);
                  if (buffer == null) {
                     MessageUtil.sendTipMessage(2242, player.getId());
                     return;
                  }

                  player.setHp(player.getFinalAttribute().getMaxHp() * buffer.getReliveRate() / 1000, 8);
                  player.setInnerPower(player.getFinalAttribute().getMaxInnerPower());
                  player.setMp(player.getFinalAttribute().getMaxMp());
                  MapManager.getInstance().afterRelive(map, player, type);
                  BufferParamBuilder paramBuilder = BufferParamBuilder.createBufferParamBuilder(buffer.getAdder(), buffer.getAdderMapObjectType(), buffer.getSkillId(), buffer.getSonBufferId(), new int[]{-buffer.getAttackRate()}, (Performer)null, buffer.getSkillId());
                  BufferManager.getInstance().addBuffer(player, paramBuilder);
                  player.bePrisoner(true);
               }

               BufferManager.getInstance().removeBufferByClass(player, new Class[]{ReliveDeathBuffer.class});
            }
         }
      }
   }

   public void changePosition(GameMap map, IMapObject object, Grid newPos) {
      Point point = object.getPoint();
      Set<Long> watchers = new HashSet();
      watchers.addAll(map.getAoi().getWatchers(point).keySet());
      watchers.addAll(map.getAoi().getWatchers(newPos).keySet());
      object.setPoint(newPos);
      map.getAoi().updateObject(object, point, newPos, false);
      if (object.getType() == 1) {
         map.getAoi().updateWatcher(object, point, newPos, 2, 2);
      }

      ResObjectChangePositionMessage res = new ResObjectChangePositionMessage();
      res.setObjectMoveResponse(ObjectMoveResponse.newBuilder().setId(object.getId()).setNewllX(newPos.getX()).setNewY(newPos.getY()).build());
      int size = watchers.size();
      if (size > 50) {
         Iterator<Long> it = watchers.iterator();

         while(it.hasNext() && watchers.size() > 50) {
            it.next();
            if (Utils.nextInt(size) >= 50) {
               it.remove();
            }
         }

         if (object.getType() == 1) {
            watchers.add(object.getId());
         }
      }

      MessageUtil.sendMsgToRids(res, watchers);
   }

   public void quitMap(IMapObject obj, boolean isOffLine) {
      GameMap map = MapManager.getInstance().getMap(obj);
      if (map != null) {
         String name = !(obj instanceof Player) && !(obj instanceof Actor) ? null : obj.getName();
         if (name != null) {
            log.info("quitMap, player = {}, 1", name);
         }

         long t = System.currentTimeMillis();
         map.removeMapObject(obj, isOffLine);
         long t2 = System.currentTimeMillis();
         if (name != null) {
         }

         map.removeFromView(obj);
         t = System.currentTimeMillis();
         if (name != null) {
         }

         if (obj.getType() == 1) {
            Player player = (Player)obj;
            if (!isOffLine) {
               player.setLastMapId(map.getId());
               player.setLastLine(map.getLine());
               player.setLastMapPoint(player.getPoint());
            }

            t2 = System.currentTimeMillis();
            List<IPlayerEnterMapScript> scripts = ScriptEngine.get1tn(IPlayerEnterMapScript.class);
            Iterator var11 = scripts.iterator();

            while(var11.hasNext()) {
               IPlayerEnterMapScript script = (IPlayerEnterMapScript)var11.next();
               script.onPlayerExitMap(map, player);
            }

            t = System.currentTimeMillis();
         }

      }
   }

   private List<BagProtos.BagItemInfo> costDeliverItem(Player player, Map<Integer, Integer> item, int mapId, int logType) {
      List<BagProtos.BagItemInfo> list = new ArrayList(item.size());
      Bag bag = player.getRole().getBag();
      Iterator var7 = item.entrySet().iterator();

      while(var7.hasNext()) {
         Map.Entry<Integer, Integer> entry = (Map.Entry)var7.next();
         if (entry != null) {
            List<BagProtos.BagItemInfo> info = bag.removeItem(logType, mapId, (Integer)entry.getKey(), (Integer)entry.getValue());
            if (info == null) {
               return null;
            }

            list.addAll(info);
         }
      }

      return list;
   }

   private int checkDeliverRequire(Player player, int[] level, Map<Integer, Integer> item, int vip, int index, boolean tip) {
      if (index != 1 && level.length == 0 && item.isEmpty() && vip == 0) {
         if (tip) {
            MessageUtil.sendTipMessage(875, player.getId());
         }

         return 0;
      } else {
         Role role = player.getRole();
         int lv = role.getLevel();
         int reincarnation = role.getReincarnation();
         int maxLevel;
         int maxRein;
         if (level.length > 0) {
            maxLevel = level[0] % 1000;
            maxRein = level[0] / 1000;
            if (reincarnation < maxRein) {
               if (tip) {
                  MessageUtil.sendTipMessage(2652, player.getId(), new Object[]{maxRein});
               }

               return 0;
            }

            if (reincarnation == maxRein && lv < maxLevel) {
               if (tip) {
                  MessageUtil.sendTipMessage(2653, player.getId(), new Object[]{maxLevel});
               }

               return 0;
            }
         }

         if (level.length > 1) {
            maxLevel = level[1] % 1000;
            maxRein = level[1] / 1000;
            if (reincarnation > maxRein) {
               if (tip) {
                  MessageUtil.sendTipMessage(2654, player.getId(), new Object[]{maxRein});
               }

               return 0;
            }

            if (reincarnation == maxRein && lv > maxLevel) {
               if (tip) {
                  MessageUtil.sendTipMessage(2655, player.getId(), new Object[]{maxLevel});
               }

               return 0;
            }
         }

         if ((role.getVipLevel() == -1 ? 0 : role.getVipLevel()) < vip && !role.isTasteVip()) {
            if (tip) {
               String strTip = String.format("FF0000vip等级达到%d才能进入[-]", vip);
               MessageUtil.sendTipMessage(2656, player.getId(), new Object[]{vip});
            }

            return 0;
         } else {
            Iterator var12 = item.entrySet().iterator();

            Map.Entry entry;
            do {
               if (!var12.hasNext()) {
                  return index;
               }

               entry = (Map.Entry)var12.next();
            } while(role.getBag().checkItemCount((Integer)entry.getKey(), (Integer)entry.getValue(), tip));

            return 0;
         }
      }
   }

   private MapProtos.TransferByDeliverExpenseState checkNeedItem(Player player, int mapId, int deliverId) {
      switch (mapId) {
         case 10041:
            Sabac2Protos.TransferSabac2PalaceState state = Sabac2Manager.getInstance().checkTransferSabac2Palace(player.getRole());
            if (deliverId == 539) {
               if (state == TransferSabac2PalaceState.NoYuanBao) {
                  return TransferByDeliverExpenseState.NoItem;
               }

               if (state == TransferSabac2PalaceState.DataError) {
                  return TransferByDeliverExpenseState.DataError;
               }
            } else if (deliverId == 538) {
               if (state == TransferSabac2PalaceState.NoYuanBao) {
                  return TransferByDeliverExpenseState.NoItem;
               }

               if (state == TransferSabac2PalaceState.DataError) {
                  return TransferByDeliverExpenseState.DataError;
               }

               return TransferByDeliverExpenseState.NoInto;
            }

            return TransferByDeliverExpenseState.NeedItem;
         default:
            return TransferByDeliverExpenseState.NeedItem;
      }
   }

   public void getMiniMapInfo(Player player) {
      GameMap map = MapManager.getInstance().getMap(player);
      if (map != null) {
         ResGetMiniMapInfoMessage msg = new ResGetMiniMapInfoMessage();
         MapProtos.GetMiniMapInfoResponse.Builder builder = GetMiniMapInfoResponse.newBuilder();
         Iterator var5 = map.getRandomBoxMap().values().iterator();

         while(var5.hasNext()) {
            RandomBox box = (RandomBox)var5.next();
            builder.addRandomBox(box.toRoundRandomBox());
         }

         MapProtos.BossInfo.Builder bld;
         if (map instanceof NewUnionBossInstance) {
            NewUnionBossInstance instance = (NewUnionBossInstance)map;
            UnionBossConfig unionBossConfig = (UnionBossConfig)ConfigDataManager.getInstance().getById(UnionBossConfig.class, instance.getCurrentId());
            if (unionBossConfig != null && instance.getUnion() != null && UnionManager.getInstance().newUnionBossIsOpen(instance.getUnion())) {
               bld = BossInfo.newBuilder().setBossId(unionBossConfig.getMId()).setBossHP(instance.bossIsAlive() ? 1 : 0).setX(unionBossConfig.getBornX()).setY(unionBossConfig.getBornY());
               builder.addBosses(bld);
            }
         } else {
            var5 = map.getMonsterMap().values().iterator();

            label54:
            while(true) {
               Monster monster;
               do {
                  do {
                     if (!var5.hasNext()) {
                        break label54;
                     }

                     monster = (Monster)var5.next();
                  } while(monster.getType() != 2);
               } while(monster.getMonsterType() != 2 && monster.getMonsterType() != 10 && monster.getMonsterType() != 13);

               bld = BossInfo.newBuilder().setBossId(monster.getConfigId()).setBossHP(monster.getHp());
               if (monster.getBirthPoint() != null) {
                  bld.setX(monster.getBirthPoint().getX()).setY(monster.getBirthPoint().getY());
                  if (monster.getMonsterType() == 2) {
                     builder.addBosses(bld);
                  } else {
                     builder.addElites(bld);
                  }
               }
            }
         }

         msg.setGetMiniMapInfoResponse(builder.build());
         MessageUtil.sendMsg(msg, player.getId());
      }
   }

   public void afterRelive(GameMap map, Player player, MapProtos.ReliveType type) {
      log.info("afterRelive() player={}, id={}, type={}", new Object[]{player.getName(), player.getId(), type});
      if (map instanceof GameInstance) {
         GameInstance instance = (GameInstance)map;
         instance.getPlayerRelivedTimes().add(player.getId());
      }

      ResReliveMessage msg = new ResReliveMessage();
      msg.setReliveResponse(ReliveResponse.newBuilder().setNewllX(player.getPoint().getX()).setNewY(player.getPoint().getY()).setId(player.getId()).setHp(player.getHp()).setType(type).setMapId(player.getMapId()).setInnerPower(player.getInnerPower()).setMp(player.getMp()).build());
      MessageUtil.sendRoundMessage(msg, player);
      int recruitPetId = player.getRole().getRecruitPetId();
      if (recruitPetId != 0) {
         boolean find = false;
         Iterator var7 = player.getPets().iterator();

         while(var7.hasNext()) {
            Pet pet = (Pet)var7.next();
            if (pet instanceof GuardPet) {
               find = true;
               break;
            }
         }

         if (find) {
            return;
         }

         GuardPet pet = MapObjectFactory.createGuardPet(recruitPetId, player, player.getRole().getRecruitSkillLevel(), player.getRole().getRecruitSkillId());
         player.getPets().add(pet);
         map.putMapObject(pet);
         Point point = player.getPoint();
         Grid grid = map.randomStandPoint(player, point.getX(), point.getY(), 1, 20, false);
         pet.setPoint(grid);
         map.addToView(pet);
      }

      if (type != ReliveType.Buffer && type != ReliveType.ReliveBuffer) {
         HSMonsterManager.getInstance().reliveHSMonsterList(player, 1);
         HSMonsterManager.getInstance().updateCanEnterFightOrFitToEnter(player);
         HSMonsterManager.getInstance().recoverMaxHpHSMonsterList(player);
      }

   }

   public void ChangeMapEventState(GameMap map, boolean open) {
      map.setMapEventOpen(open);
      ResMapEventChangeMessage res = new ResMapEventChangeMessage();
      MapProtos.MapEventStateChangeResponse.Builder builder = MapEventStateChangeResponse.newBuilder();
      builder.setIsOpen(map.getMapEventOpen());
      res.setMapEventStateChangeResponse(builder.build());
      MessageUtil.sendMsgToRids(res, map.getPlayerMap().keySet());
   }

   public boolean shareBossOpen() {
      int effect = RoleManager.getSundryConfigEffect(2022, 0, 0);
      if (effect == 1) {
         return false;
      } else {
         long now = System.currentTimeMillis();
         int[] effects = RoleManager.getSundryConfigEffects(2020, new int[]{4, 6});
         long closeStartTime = TimeUtil.getTodayTime(effects[0], 0, 0, 0);
         long closeEndTime = TimeUtil.getTodayTime(effects[1], 0, 0, 0);
         return now < closeStartTime || now > closeEndTime;
      }
   }

   public boolean checkLastEnterShareBossTime(Player player) {
      if (!ClientMapManger.getInstance().isShareBossOpen(player.getServerId())) {
         MessageUtil.sendTipMessage("[ff0000]跨服boss未开放，请您稍等段时间[-]", player.getId());
         log.info("共享boss当前服未开放 serverId:{}", player.getServerId());
         return false;
      } else {
         Map<Long, Long> enterShareBossTimeMap = MapManager.getInstance().getEnterShareBossTimeMap();
         long roleId = player.getId();
         Long lastEnterTime = (Long)enterShareBossTimeMap.getOrDefault(roleId, 0L);
         long current = System.currentTimeMillis();
         int enterDiffTime = (int)((current - lastEnterTime) / 1000L);
         int timeLimit = RoleManager.getSundryConfigEffect(2023, 0, 60);
         int diffTime = timeLimit - enterDiffTime;
         if (diffTime > 0) {
            MessageUtil.sendTipMessage(String.format("[ff0000]%s秒后方可进入[-]", diffTime), roleId);
            return false;
         } else {
            return true;
         }
      }
   }

   public void updateEnterShareBossTime(Player player) {
      if (player != null) {
         long roleId = player.getId();
         Map<Long, Long> enterShareBossTimeMap = MapManager.getInstance().getEnterShareBossTimeMap();
         long enterTime = System.currentTimeMillis();
         enterShareBossTimeMap.put(roleId, enterTime);
         boolean gameServer = ApplicationContext.getInstance().isGameServer();
         boolean shareServerOr2 = ApplicationContext.getInstance().isShareServerOr2();
         ReqGameCommonMessage reqGameCommonMessage = new ReqGameCommonMessage();
         reqGameCommonMessage.setCommonInfo(CommonInfo.newBuilder().setType(26).setData(ApplicationContext.getInstance().getServerType()).setData64(roleId).setData641(enterTime).build());
         if (gameServer) {
            MessageUtil.sendMsgToShare(reqGameCommonMessage);
            MessageUtil.sendMsgToShare2(reqGameCommonMessage);
         } else if (shareServerOr2) {
            MessageUtil.sendMsgToGame(reqGameCommonMessage, player.getServerId());
         } else {
            log.error("这就有点问题了");
         }

      }
   }

   public void updateEnterShareBossTime(long roleId, long enterTime) {
      Map<Long, Long> enterShareBossTimeMap = MapManager.getInstance().getEnterShareBossTimeMap();
      enterShareBossTimeMap.put(roleId, enterTime);
   }
}
